@@ -3,7 +3,6 @@ use strict; use warnings;
 
 use Carp;
 use DBI;
-use Data::Page;
 
 use overload
     '""'     => sub { shift->stringify_self },
@@ -42,6 +41,7 @@ sub construct {
     my $data  = shift;
 
     # see unless block on line 526 of viperl Class::DBI for cached alternative
+    # that line is in the _init method
     my $obj   = bless {}, $class;
 
     my @cols  = keys %{ $data };
@@ -216,7 +216,10 @@ sub create {
     # fill in primary_key if needed (it's usually needed)
     my $primary   = $class->get_primary_col();
 
-    unless ( defined $value_for->{ $primary } ) {
+    if ( not defined $value_for->{ $primary }
+            and
+         $class->can( 'get_sequence_name' )
+    ) {
         $value_for->{ $primary } = $class->_next_primary_key();
     }
 
@@ -393,13 +396,13 @@ Gantry::Utils::Model - a general purpose Object Relational Model base class
     sub get_table_name     { return 'your_table';                           }
     sub get_sequence_name  { return 'your_table_seq';                       }
     sub get_primary_col    { return 'id';                                   }
-    sub get_quoted_cols    { return { text_col => 1, other_text_col => 1 }; }
     sub get_essential_cols { return 'id, text_col';                         }
 
     sub get_primary_key    { goto &get_id;                                  }
 
     sub set_id             { croak "Can't change primary key";              }
     sub get_id             { return $_->[0]{id};                            }
+    sub quote_id           { return $_[1];                                  }
 
     sub set_text_col       {
         my $self  = shift;
@@ -410,6 +413,9 @@ Gantry::Utils::Model - a general purpose Object Relational Model base class
         return $value;
     }
     sub get_text_col       { return $_->[0]{text_col};                      }
+    sub quote_text_col     {
+        return ( defined $_[1] ) ? "'$_[1]'" : 'NULL';
+    }
 
     sub set_other_text_col {
         my $self  = shift;
@@ -427,25 +433,29 @@ Gantry::Utils::Model - a general purpose Object Relational Model base class
         }
         return $self->{other_text_col};
     }
+    sub quote_other_text_col     {
+        return ( defined $_[1] ) ? "'$_[1]'" : 'NULL';
+    }
 
 =head1 DESCRIPTION
 
 This module is a Class::DBI replacement.  Its goal is to reduce the mystery
-in in the internals of that module, while still providing most of its
+in the internals of that module, while still providing most of its
 functionality.  You'll notice that the inheriting class has a lot more code
 than a Class::DBI subclass would.  This is because we use Bigtop to
 generate the subclasses.  Thus, we don't care so much about the volume of
-code.  The result is code which is easy to read, understand, and modify.
+code.  The result is code which is easy to read, understand, override
+and/or modify.
 
 =head1 RATIONALE
 
-Class::DBI and its cousins provide a beautiful API for client code.  By
+Class::DBI and its cousins provide beautiful APIs for client code.  By
 implementing straightforward database row to Perl object correspondence,
 they save a lot of mental effort when writing most applications.
 
 They do have drawbacks.  My premise is that most of these drawbacks stem
 from a single fundamental design descision.  Perl's traditional Object
-Relation Mappers (ORMs) do a lot of work at run time.  Namely, they build
+Relation Mappers (ORMs) do a lot of work at run time.  For instance, they build
 accessors at run time.  When I first started using them, I thought this
 was gorgeous.  Class::DBI::mysql was one of my favorite modules.  I
 bought the promise of a future where all you had to say was something like
@@ -457,7 +467,7 @@ bought the promise of a future where all you had to say was something like
 
 and the whole somedb database would be mapped without another word.  Each table
 would become a class under MyModel with an accessor for each column.  Then
-I could create, retrieve, update, and delete to my hearts content while
+I could create, retrieve, update, and delete to my heart's content while
 beholding the power of Perl.
 
 The problem is that use statements like the above example require extreme
@@ -480,12 +490,13 @@ In order to achieve transperency, it is necessary to have more code in the
 subclasses.  This is really why the magical schemes sprang up.  But,
 recently I have been working on generation of code.  This amounts to the
 same thing, but it happens ahead of time.  So, instead of code being
-generated by magic during run time, my code is generated by simple grammar
+generated by magic during run time, my code is generated by grammar
 based parsing before compile time.  The generator in question is bigtop
 which can build a completely funcational web app from a description of
-its data models and controllers.  Then, when a programmer wonders what
+its data model and controllers.  Then, when a programmer wonders what
 the model is up to, she has a set of simple modules which explicitly
-show what is going on.
+show what is going on.  To make change, she may add methods or override
+the existing generated ones.
 
 =head1 METHODS PROVIDED BY THIS MODULE
 
@@ -541,17 +552,39 @@ arguments), those arguments are forwarded to search.
 =item search
 
 Class method.  Similar to search in Class::DBI.  Call with the key/value
-pairs you want to match in a simple list.  Add a single hash reference as
-the last parameter if you like.  Currently that hash reference may only
-contain an order_by key.  Its value is used to fill in the blank
-'ORDER BY ___'.
+pairs you want to match in a simple list.
 
 Returns a list of objects one each for every row that matched the search
 criterion.
 
+Add a single hash reference as the last parameter if you like.  That hash
+reference may only contain these keys:
+
+=over 4
+
+=item order_by
+
+Asks for an ORDER BY clause, the value is used literally to fill in the
+blank in 'ORDER BY ___'.
+
+=item rows
+
+Indicates that you want paging.  The value is the number of rows per page.
+There is no default, since the absence of this key is taken to mean you don't
+want paging.
+
+=item page
+
+Ignored unless rows is supplied.  Defaults to 1.  This is the page number
+to retrieve.
+
+=back
+
 =item page
 
 A synonymn for search to better match the Class::DBI::Sweet API.
+Note that you must set the rows key in the hash reference passed as the last
+argument.  You may also set the page key.  See above.
 
 =item lazy_fetch
 
@@ -562,13 +595,15 @@ row in the underlying table.
 =item create
 
 Class method.  Call with a hash reference whose keys are the column names
-you want to populate.  Any value will be quoted if its value in the
-get_quoted_cols hash is defined.
+you want to populate.  The value will be quoted for you according to the
+corresponding quote_* method in the subclass.
 
 =item _next_primary_key
 
 Class method.  Returns the next value of the sequence associated with
-the underlying table.
+the underlying table.  This is not reproduceable, it actually increments
+the sequence.  It only works if the database is using a sequence for the
+table and the model implements get_sequence_name.
 
 =item find_or_create
 
@@ -650,12 +685,6 @@ automatically during retrieve, search, etc.
 We assume that each table has a unique primary key (though we assume
 nothing about its name).  Return the name of that column.
 
-=item get_quoted_cols
-
-Return a hash reference whose keys are the names of columns whose values should
-be single quoted in SQL statements.  The values are unimportant, so long
-as they are defined.
-
 =item get_primary_key
 
 An instance method.  Return the value of the primary key for the invocant.
@@ -690,6 +719,15 @@ column.  Example:
         return $self->{amount};
     }
 
+=item quote_COL_NAME
+
+Called as a class or instance method with one argument.  Take that argument,
+hold it up to the light, examine in detail.  Then return something that has
+the same value properly quoted for SQL.
+
+Note that you should not look in the object, even if one is used as the
+invocant.  Always only work on the other argument.
+
 =item COL_NAME (completely optional)
 
 Provide one of these for each column only if you like.  Dispatch to get_ and
